==> 1.6.1&file=out_yasapi-1.6.1-src.tar.gz: out_yasapi-1.6.1/out_yasapi/yasapi_device.c <==
/*
 * yasapi_plugin_device.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

int PlayerDeviceCreate(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
#if defined (YASAPI_DEBUG) // {
  int debug=pPlayer->options.common.nDebug;
#endif // }
  PWSTR pstrId=pPlayer->options.common.szId;
  IMMDeviceEnumerator *pEnumerator=pPlayer->run.pEnumerator;
  IMMDevice *pDevice;
  HRESULT hr;

  DPRINTF(debug,0,"  > %s <\n",__func__);

  if (!pstrId||!pstrId[0]) {
    ///////////////////////////////////////////////////////////////////////////
    hr=pEnumerator->lpVtbl->GetDefaultAudioEndpoint(pEnumerator,
      eRender,              // [in]  EDataFlow dataFlow,
      eMultimedia,          // [in]  ERole     role,
      &pDevice              // [out] IMMDevice **ppDevice
    );

    if (FAILED(hr)) {
      DERROR(debug,E_POINTER,hr);
      DERROR(debug,E_INVALIDARG,hr);
      DERROR(debug,E_NOTFOUND,hr);
      DERROR(debug,E_OUTOFMEMORY,hr);
      DUNKNOWN(debug,hr);
      DMESSAGE(debug,"getting the default audio device");
      goto device;
    }

    DPRINTF(debug,0,"  got the the default audio device\n");
    pPlayerDevice->pDevice=pDevice;

    ///////////////////////////////////////////////////////////////////////////
    hr=pDevice->lpVtbl->GetId(pDevice,
      &pstrId               // [out] LPWSTR *ppstrId
    );

    if (FAILED(hr)) {
      DERROR(debug,E_OUTOFMEMORY,hr);
      DERROR(debug,E_POINTER,hr);
      DUNKNOWN(debug,hr);
      DMESSAGE(debug,"getting endpoint ID string");
      goto id;
    }
  
    DPUTS(debug,0,"  got the endpoint ID string\n");
    wcscpy(pPlayerDevice->szId,pstrId);
    CoTaskMemFree(pstrId);
  }
  else {
    pPlayerDevice->pDevice=NULL;
    wcscpy(pPlayerDevice->szId,pstrId);
  }

  return 0;
id:
  pDevice->lpVtbl->Release(pDevice);
device:
  return -1;
}

int PlayerDeviceDestroy(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
#if defined (YASAPI_DEBUG) // {
  int debug=pPlayer->options.common.nDebug;
#endif // }
  IMMDevice *pDevice=pPlayerDevice->pDevice;

  DPRINTF(debug,0,"  > %s <\n",__func__);

  if (pDevice) {
    DPUTS(debug,0,"  destroying device\n");
    pDevice->lpVtbl->Release(pDevice);
  }

  ZeroMemory(pPlayerDevice, sizeof *pPlayerDevice);

  return 0;
}

int PlayerDeviceGet(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
#if defined (YASAPI_DEBUG) // {
  int debug=pPlayer->options.common.nDebug;
#endif // }
  LPWSTR pstrId=pPlayer->device.szId;
  IMMDeviceEnumerator *pEnumerator=pPlayer->run.pEnumerator;
  IMMDevice *pDevice;
  HRESULT hr;

  DPRINTF(debug,0,"  > %s <\n",__func__);

  if(!pPlayerDevice->pDevice) {
    ///////////////////////////////////////////////////////////////////////////
    hr=pEnumerator->lpVtbl->GetDevice(pEnumerator,
      pstrId,               // [in]  LPCWSTR   pwstrId,
      &pDevice              // [out] IMMDevice **ppDevice
    );

    if (FAILED(hr)) {
      DERROR(debug,E_POINTER,hr);
      DERROR(debug,E_NOTFOUND,hr);
      DERROR(debug,E_OUTOFMEMORY,hr);
      DUNKNOWN(debug,hr);
      DWMESSAGEV(debug,L"getting the audio device %s",pstrId);
      goto device;
    }

    DPRINTF(debug,0,"  got the the audio device\n");
    pPlayerDevice->pDevice=pDevice;
  }

  return 0;
device:
  return -1;
}

int PlayerDeviceCreateV(Player *pPlayer, va_list ap)
{
  PlayerDevice *pPlayerDevice=va_arg(ap,PlayerDevice *);

  return PlayerDeviceCreate(pPlayerDevice,pPlayer);
}

int PlayerDeviceDestroyV(Player *pPlayer, va_list ap)
{
  PlayerDevice *pPlayerDevice=va_arg(ap,PlayerDevice *);

  return PlayerDeviceDestroy(pPlayerDevice,pPlayer);
}

==> 1.6.12&file=out_yasapi-1.6.12-src.tar.gz: out_yasapi-1.6.12/out_yasapi/yasapi_device.c <==
/*
 * yasapi_plugin_device.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

int PlayerDeviceCreate(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
  PWSTR pstrId=pPlayer->options.common.szId;
  IMMDeviceEnumerator *pEnumerator=pPlayer->run.pEnumerator;
  IMMDevice *pDevice;
  HRESULT hr;

  DPRINTF(0,"  > %s <\n",__func__);

  if (!pstrId||!pstrId[0]) {
    ///////////////////////////////////////////////////////////////////////////
    hr=pEnumerator->lpVtbl->GetDefaultAudioEndpoint(pEnumerator,
      eRender,              // [in]  EDataFlow dataFlow,
      eMultimedia,          // [in]  ERole     role,
      &pDevice              // [out] IMMDevice **ppDevice
    );

    if (FAILED(hr)) {
      DERROR(E_POINTER,hr);
      DERROR(E_INVALIDARG,hr);
      DERROR(E_NOTFOUND,hr);
      DERROR(E_OUTOFMEMORY,hr);
      DUNKNOWN(hr);
      DMESSAGE("getting the default audio device");
      goto device;
    }

    DPRINTF(0,"  got the the default audio device\n");
    pPlayerDevice->pDevice=pDevice;

    ///////////////////////////////////////////////////////////////////////////
    hr=pDevice->lpVtbl->GetId(pDevice,
      &pstrId               // [out] LPWSTR *ppstrId
    );

    if (FAILED(hr)) {
      DERROR(E_OUTOFMEMORY,hr);
      DERROR(E_POINTER,hr);
      DUNKNOWN(hr);
      DMESSAGE("getting endpoint ID string");
      goto id;
    }
  
    DPUTS(0,"  got the endpoint ID string\n");
    wcscpy(pPlayerDevice->szId,pstrId);
    CoTaskMemFree(pstrId);
  }
  else {
    pPlayerDevice->pDevice=NULL;
    wcscpy(pPlayerDevice->szId,pstrId);
  }

  return 0;
id:
  pDevice->lpVtbl->Release(pDevice);
device:
  return -1;
}

int PlayerDeviceDestroy(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
  IMMDevice *pDevice=pPlayerDevice->pDevice;

  DPRINTF(0,"  > %s <\n",__func__);

  if (pDevice) {
    DPUTS(0,"  destroying device\n");
    pDevice->lpVtbl->Release(pDevice);
  }

  ZeroMemory(pPlayerDevice, sizeof *pPlayerDevice);

  return 0;
}

int PlayerDeviceGet(PlayerDevice *pPlayerDevice, Player *pPlayer)
{
  LPWSTR pstrId=pPlayer->device.szId;
  IMMDeviceEnumerator *pEnumerator=pPlayer->run.pEnumerator;
  IMMDevice *pDevice;
  HRESULT hr;

  DPRINTF(0,"  > %s <\n",__func__);

  if(!pPlayerDevice->pDevice) {
    ///////////////////////////////////////////////////////////////////////////
    hr=pEnumerator->lpVtbl->GetDevice(pEnumerator,
      pstrId,               // [in]  LPCWSTR   pwstrId,
      &pDevice              // [out] IMMDevice **ppDevice
    );

    if (FAILED(hr)) {
      DERROR(E_POINTER,hr);
      DERROR(E_NOTFOUND,hr);
      DERROR(E_OUTOFMEMORY,hr);
      DUNKNOWN(hr);
      DWMESSAGEV(L"getting the audio device %s",pstrId);
      goto device;
    }

    DPRINTF(0,"  got the the audio device\n");
    pPlayerDevice->pDevice=pDevice;
  }

  return 0;
device:
  return -1;
}

int PlayerDeviceCreateV(Player *pPlayer, Request *pRequest)
{
  PlayerDevice *pPlayerDevice=va_arg(pRequest->ap,PlayerDevice *);

  return PlayerDeviceCreate(pPlayerDevice,pPlayer);
}

int PlayerDeviceDestroyV(Player *pPlayer, Request *pRequest)
{
  PlayerDevice *pPlayerDevice=va_arg(pRequest->ap,PlayerDevice *);

  return PlayerDeviceDestroy(pPlayerDevice,pPlayer);
}
